@@ -11,22 +11,25 @@ class LibraryInfo(object):
         self.included = []
         self.required = []
 
+def _required(required_list, req):
+    if req not in required_list:
+        required_list.append(req)
+        for r in getRequired(req):
+            _required(required_list, r)
 
 def need(library_name):
     registrar = getUtility(IHeadIncludeRegistration)
     if registrar:
-        myList = [library_name]
+        myList = []
         try:
-            required = getRequired(library_name)
+            _required(myList, library_name)
         except KeyError:
             raise RuntimeError('Unknown resource library: %s' % library_name)
-        for lib in required:
-            myList.append(lib)
         myList.reverse()
         for lib in myList:
             included = getIncluded(lib)
             for file_name in included:
-                url = '/@@/%s/%s' % (library_name,file_name)
+                url = '/@@/%s/%s' % (lib, file_name)
                 registrar.register(url)
 
 def getRequired(name):
@@ -34,3 +37,12 @@ def getRequired(name):
 
 def getIncluded(name):
     return library_info[name].included
+
+# register cleanup with zope.testing if available
+try:
+    from zope.testing.cleanup import addCleanUp
+except ImportError:
+    pass
+else:
+    addCleanUp(lambda: library_info.clear())
+    del addCleanUp
